'use strict';

Object.defineProperty(exports, "__esModule", {
    value: true
});

var _parseColor = require('parse-color');

var _parseColor2 = _interopRequireDefault(_parseColor);

var _parseCssDimension = require('parse-css-dimension');

var _parseCssDimension2 = _interopRequireDefault(_parseCssDimension);

var _nodeSass = require('node-sass');

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var unitTypes = ['length', 'angle', 'resolution', 'frequency', 'time'];

function isPlainObject(object) {
    return object instanceof Object && !Array.isArray(object) && typeof object !== 'function';
}

function isColor(value) {
    return typeof (0, _parseColor2.default)(value).rgba !== 'undefined';
}

function parseValueToStringOrNumber(value) {
    var resolvedValue = null;

    try {
        var resolvedUnitValue = (0, _parseCssDimension2.default)(value);

        if (unitTypes.indexOf(resolvedUnitValue.type) !== -1) {
            resolvedValue = new _nodeSass.types.Number(resolvedUnitValue.value, resolvedUnitValue.unit);
        } else if (resolvedUnitValue.type === 'percentage') {
            resolvedValue = new _nodeSass.types.Number(resolvedUnitValue.value, '%');
        } else {
            resolvedValue = new _nodeSass.types.String(resolvedUnitValue.value);
        }
    } catch (error) {
        // eslint-disable-line no-unused-vars
        resolvedValue = new _nodeSass.types.String(value);
    }

    return resolvedValue;
}

function parseValueToColor(value) {
    var resolvedColorValue = (0, _parseColor2.default)(value).rgba;

    return new _nodeSass.types.Color(resolvedColorValue[0], resolvedColorValue[1], resolvedColorValue[2], resolvedColorValue[3]);
}

function arrayToList(arr) {
    var length = arr.length;

    var data = new _nodeSass.types.List(length);

    for (var i = 0; i < length; i++) {
        // eslint-disable-next-line no-use-before-define
        data.setValue(i, setJsonValueToSassValue(arr[i]));
    }

    return data;
}

function objectToMap(obj) {
    var _Object$keys = Object.keys(obj),
        length = _Object$keys.length;

    var data = new _nodeSass.types.Map(length);
    var i = 0;

    for (var prop in obj) {
        // eslint-disable-next-line no-prototype-builtins
        if (obj.hasOwnProperty(prop)) {
            // eslint-disable-next-line no-use-before-define
            data.setKey(i, setJsonValueToSassValue(prop));
            // eslint-disable-next-line no-use-before-define
            data.setValue(i, setJsonValueToSassValue(obj[prop]));
            i++;
        }
    }

    return data;
}

function setJsonValueToSassValue(value) {
    var resolvedValue = _nodeSass.types.Null.NULL;

    if (Array.isArray(value)) {
        resolvedValue = arrayToList(value);
    } else if (isPlainObject(value)) {
        resolvedValue = objectToMap(value);
    } else if (isColor(value)) {
        resolvedValue = parseValueToColor(value);
    } else if (typeof value === 'string') {
        resolvedValue = parseValueToStringOrNumber(value);
    } else if (typeof value === 'number') {
        resolvedValue = new _nodeSass.types.Number(value);
    } else if (typeof value === 'boolean') {
        resolvedValue = value ? _nodeSass.types.Boolean.TRUE : _nodeSass.types.Boolean.FALSE;
    }

    return resolvedValue;
}

exports.default = setJsonValueToSassValue;