'use strict';

Object.defineProperty(exports, "__esModule", {
    value: true
});

var _rgbHex = require('rgb-hex');

var _rgbHex2 = _interopRequireDefault(_rgbHex);

var _lodash = require('lodash.round');

var _lodash2 = _interopRequireDefault(_lodash);

var _shortenCssHex = require('shorten-css-hex');

var _shortenCssHex2 = _interopRequireDefault(_shortenCssHex);

var _nodeSass = require('node-sass');

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _toConsumableArray(arr) { if (Array.isArray(arr)) { for (var i = 0, arr2 = Array(arr.length); i < arr.length; i++) { arr2[i] = arr[i]; } return arr2; } else { return Array.from(arr); } }

function listToArray(list, opts) {
    var length = list.getLength();
    var data = [];

    for (var i = 0; i < length; i++) {
        // eslint-disable-next-line no-use-before-define
        var value = getJsonValueFromSassValue(list.getValue(i), opts);

        data.push(value);
    }

    return data;
}

function mapToObject(map, opts) {
    var length = map.getLength();
    var data = {};

    for (var i = 0; i < length; i++) {
        var key = map.getKey(i).getValue();

        // eslint-disable-next-line no-use-before-define
        data[key] = getJsonValueFromSassValue(map.getValue(i), opts);
    }

    return data;
}

function getJsonValueFromSassValue(value, opts) {
    var resolvedValue = null;
    var rgbValue = [];

    if (value instanceof _nodeSass.types.List) {
        resolvedValue = listToArray(value, opts);
    } else if (value instanceof _nodeSass.types.Map) {
        resolvedValue = mapToObject(value, opts);
    } else if (value instanceof _nodeSass.types.Color) {
        rgbValue = [value.getR(), value.getG(), value.getB()];

        var alphaValue = value.getA();

        if (alphaValue === 1) {
            resolvedValue = (0, _shortenCssHex2.default)('#' + _rgbHex2.default.apply(undefined, _toConsumableArray(rgbValue)));
        } else {
            resolvedValue = 'rgba(' + rgbValue.join(',') + ',' + alphaValue + ')';
        }
    } else if (value instanceof _nodeSass.types.Number) {
        if (value.getUnit() !== '') {
            resolvedValue = String((0, _lodash2.default)(Number(value.getValue()), opts.precision) + value.getUnit());
        } else {
            resolvedValue = (0, _lodash2.default)(Number(value.getValue()), opts.precision);
        }
    } else if (!(value instanceof _nodeSass.types.Null)) {
        resolvedValue = value.getValue();
    }

    return resolvedValue;
}

exports.default = getJsonValueFromSassValue;